package com.fjhx.common.service.art.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.entity.art.dto.ArtCopyrightDto;
import com.fjhx.common.entity.art.dto.ArtCopyrightSelectDto;
import com.fjhx.common.entity.art.po.ArtCopyright;
import com.fjhx.common.entity.art.vo.ArtCopyrightVo;
import com.fjhx.common.mapper.art.ArtCopyrightMapper;
import com.fjhx.common.service.art.ArtCopyrightService;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 美术版权 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-24
 */
@Service
public class ArtCopyrightServiceImpl extends ServiceImpl<ArtCopyrightMapper, ArtCopyright> implements ArtCopyrightService {

    @Autowired
    private ISysDeptService deptService;

    @Override
    public Page<ArtCopyrightVo> getPage(ArtCopyrightSelectDto dto) {
        IWrapper<ArtCopyright> wrapper = getWrapper();

        //关键字检索
        wrapper.keyword(dto.getKeyword(), new SqlField("ac", ArtCopyright::getName), new SqlField("ac", ArtCopyright::getCode));
        wrapper.orderByDesc("ac", ArtCopyright::getCreateTime);

        wrapper.orderByDesc("ac", ArtCopyright::getId);
        Page<ArtCopyrightVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ArtCopyrightVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        //获取归属公司信息
        Map<Long, String> deptMap = deptService.list().stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        //赋值数据
        for (ArtCopyrightVo record : records) {
            //赋值归属公司信息
            record.setCompanyName(deptMap.get(record.getCompanyId()));
        }

        return page;
    }

    @Override
    public ArtCopyrightVo detail(Long id) {
        ArtCopyright ArtCopyright = this.getById(id);
        ArtCopyrightVo result = BeanUtil.toBean(ArtCopyright, ArtCopyrightVo.class);
        return result;
    }

    @Override
    public void add(ArtCopyrightDto artCopyrightDto) {
        this.save(artCopyrightDto);
    }

    @Override
    public void edit(ArtCopyrightDto artCopyrightDto) {
        this.updateById(artCopyrightDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
