package com.fjhx.common.service.contractor.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.entity.contractor.dto.ContractorDto;
import com.fjhx.common.entity.contractor.dto.ContractorSelectDto;
import com.fjhx.common.entity.contractor.po.Contractor;
import com.fjhx.common.entity.contractor.vo.ContractorVo;
import com.fjhx.common.mapper.contractor.ContractorMapper;
import com.fjhx.common.service.contractor.ContractorService;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 承包商 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-24
 */
@Service
public class ContractorServiceImpl extends ServiceImpl<ContractorMapper, Contractor> implements ContractorService {

    @Autowired
    private ISysDeptService deptService;

    @Override
    public Page<ContractorVo> getPage(ContractorSelectDto dto) {
        IWrapper<Contractor> wrapper = getWrapper();

        //关键字检索
        wrapper.keyword(dto.getKeyword(),
                new SqlField("c", Contractor::getName),
                new SqlField("c", Contractor::getCode),
                new SqlField("c", Contractor::getAccountName),
                new SqlField("c", Contractor::getAccountBank),
                new SqlField("c", Contractor::getAccountNumber)
        );
        wrapper.orderByDesc("c", Contractor::getCreateTime);

        wrapper.orderByDesc("c", Contractor::getId);
        Page<ContractorVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        List<ContractorVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        //获取归属公司信息
        Map<Long, String> deptMap = deptService.list().stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        //赋值数据
        for (ContractorVo record : records) {
            //赋值归属公司信息
            record.setCompanyName(deptMap.get(record.getCompanyId()));
        }

        return page;
    }

    @Override
    public ContractorVo detail(Long id) {
        Contractor Contractor = this.getById(id);
        ContractorVo result = BeanUtil.toBean(Contractor, ContractorVo.class);
        return result;
    }

    @Override
    public void add(ContractorDto contractorDto) {
        this.save(contractorDto);
    }

    @Override
    public void edit(ContractorDto contractorDto) {
        this.updateById(contractorDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
