package com.fjhx.common.service.patent.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.entity.patent.dto.PatentDto;
import com.fjhx.common.entity.patent.dto.PatentSelectDto;
import com.fjhx.common.entity.patent.po.Patent;
import com.fjhx.common.entity.patent.vo.PatentVo;
import com.fjhx.common.mapper.patent.PatentMapper;
import com.fjhx.common.service.patent.PatentService;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 专利 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-24
 */
@Service
public class PatentServiceImpl extends ServiceImpl<PatentMapper, Patent> implements PatentService {

    @Autowired
    private ISysDeptService deptService;

    @Override
    public Page<PatentVo> getPage(PatentSelectDto dto) {
        IWrapper<Patent> wrapper = getWrapper();

        //关键字检索
        wrapper.keyword(dto.getKeyword(), new SqlField("p", Patent::getName), new SqlField("p", Patent::getCode));
        wrapper.orderByDesc("p", Patent::getCreateTime);

        wrapper.orderByDesc("p", Patent::getId);
        Page<PatentVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        List<PatentVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        //获取归属公司信息
        Map<Long, String> deptMap = deptService.list().stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        //赋值数据
        for (PatentVo record : records) {
            //赋值归属公司信息
            record.setCompanyName(deptMap.get(record.getCompanyId()));
        }

        return page;
    }

    @Override
    public PatentVo detail(Long id) {
        Patent Patent = this.getById(id);
        PatentVo result = BeanUtil.toBean(Patent, PatentVo.class);
        return result;
    }

    @Override
    public void add(PatentDto patentDto) {
        this.save(patentDto);
    }

    @Override
    public void edit(PatentDto patentDto) {
        this.updateById(patentDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
