package com.fjhx.common.service.trademark.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.entity.trademark.dto.TrademarkDto;
import com.fjhx.common.entity.trademark.dto.TrademarkSelectDto;
import com.fjhx.common.entity.trademark.po.Trademark;
import com.fjhx.common.entity.trademark.vo.TrademarkVo;
import com.fjhx.common.mapper.trademark.TrademarkMapper;
import com.fjhx.common.service.trademark.TrademarkService;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 商标 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-24
 */
@Service
public class TrademarkServiceImpl extends ServiceImpl<TrademarkMapper, Trademark> implements TrademarkService {

    @Autowired
    private ISysDeptService deptService;

    @Override
    public Page<TrademarkVo> getPage(TrademarkSelectDto dto) {
        IWrapper<Trademark> wrapper = getWrapper();

        //关键字检索
        wrapper.keyword(dto.getKeyword(), new SqlField("t", Trademark::getName), new SqlField("t", Trademark::getCode));
        wrapper.orderByDesc("t", Trademark::getCreateTime);

        wrapper.orderByDesc("t", Trademark::getId);
        Page<TrademarkVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<TrademarkVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        //获取归属公司信息
        Map<Long, String> deptMap = deptService.list().stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        //赋值数据
        for (TrademarkVo record : records) {
            //赋值归属公司信息
            record.setCompanyName(deptMap.get(record.getCompanyId()));
        }

        return page;
    }

    @Override
    public TrademarkVo detail(Long id) {
        Trademark Trademark = this.getById(id);
        TrademarkVo result = BeanUtil.toBean(Trademark, TrademarkVo.class);
        return result;
    }

    @Override
    public void add(TrademarkDto trademarkDto) {
        this.save(trademarkDto);
    }

    @Override
    public void edit(TrademarkDto trademarkDto) {
        this.updateById(trademarkDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
