package com.fjhx.form.flow;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.account.entity.account.enums.PaymentStatusEnum;
import com.fjhx.account.entity.account.po.AccountPayment;
import com.fjhx.account.entity.account.po.AccountRunningWater;
import com.fjhx.account.service.account.AccountPaymentService;
import com.fjhx.account.service.account.AccountRunningWaterService;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.enums.NodeTypeEnum;
import com.fjhx.form.entity.cost.dto.CostControlDto;
import com.fjhx.form.entity.cost.po.CostControl;
import com.fjhx.form.entity.cost.po.CostControlDetail;
import com.fjhx.form.service.cost.CostControlDetailService;
import com.fjhx.form.service.cost.CostControlService;
import com.fjhx.wms.entity.stock.po.StockJournalDetails;
import com.fjhx.wms.service.stock.StockJournalDetailsService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 费控流程
 *
 * @author y
 * @since 2024-03-19
 */
@Component
public class CostControlFlow extends FlowDelegate {

    @Autowired
    private CostControlService costControlService;
    @Autowired
    private CodingRuleService codingRuleService;
    @Autowired
    private CostControlDetailService costControlDetailService;
    @Autowired
    private StockJournalDetailsService stockJournalDetailsService;
    @Autowired
    private AccountPaymentService accountPaymentService;
    @Autowired
    private AccountRunningWaterService accountRunningWaterService;
    @Autowired
    private ISysDeptService sysDeptService;

    @Override
    public String getFlowKey() {
        return "cost_control_flow";
    }

    /**
     * 发起流程
     */
    @Override
    public Long start(Long flowId, JSONObject submitData) {
        CostControlDto costControlDto = submitData.toJavaObject(CostControlDto.class);
        costControlDto.setFlowId(flowId);
        costControlDto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        costControlDto.setCode(codingRuleService.createCode(CodingRuleEnum.COST_CONTROL.getKey(), null));

        //保存数据
        costControlService.addOrEdit(costControlDto);

        //回传数据给流程引擎
        submitData.put("code", costControlDto.getCode());
        submitData.put("amount", costControlDto.getAmount());

        return costControlDto.getId();
    }

    /**
     * 结束流程
     */
    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        CostControl costControl = costControlService.getById(businessId);

        costControlService.update(q -> q
                .eq(CostControl::getId, businessId)
                .set(CostControl::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );

        List<CostControlDetail> costControlDetailList = costControlDetailService.list(q -> q.eq(CostControlDetail::getCostControlId, businessId));
        for (CostControlDetail costControlDetail : costControlDetailList) {

            //标记流水为已结算
            String stockJournalDetailsIds = costControlDetail.getStockJournalDetailsIds();
            if (ObjectUtil.isNotEmpty(stockJournalDetailsIds)) {
                stockJournalDetailsService.update(q -> q
                        .in(StockJournalDetails::getId, stockJournalDetailsIds.split(","))
                        .set(StockJournalDetails::getSettleStatus, 2));
            }
        }

        //如果是属于付款类型的，需要自动生成一条打款数据
        if (ObjectUtil.equals(costControl.getTradeType(), 20)) {
            // 添加一条付款流水
            AccountPayment payment = new AccountPayment();
            payment.setBusinessId(businessId);
            payment.setCostControlId(businessId);

            payment.setAccountManagementId(costControl.getPaymentAccountId());
            payment.setCurrency("CNY");
            payment.setStatus(PaymentStatusEnum.UNDER_REVIEW.getKey());
            payment.setAmount(BigDecimal.ZERO);
            payment.setIncomeAmount(costControl.getAmount());
            payment.setName(costControl.getAccountName());
            payment.setDataUser(costControl.getApplyUserId());
            payment.setBusinessManagementId(costControl.getPaymentAccountId());
            payment.setOpeningBank(costControl.getAccountBank());
            payment.setBusinessCurrency("CNY");
            payment.setAccountOpening(costControl.getAccountNumber());
            payment.setCompanyIds(costControl.getCompanyId());
            payment.setDepartmentId(SecurityUtils.getDeptId());

            //数据类型直接拿费控
            payment.setType(String.valueOf(costControl.getCostType()));


            //赋值付款方式 费控 是否公户转出 是 赋值bank1  否 赋值bank2  没有是否公户转出字段的话 默认bank2
            payment.setPaymentMethod("bank2");
            if (ObjectUtil.isNotEmpty(costControl.getIsPublicTransfer()) && costControl.getIsPublicTransfer().equals(1)) {
                payment.setPaymentMethod("bank1");
            }


            accountPaymentService.save(payment);
        } else {
            //添加一条资金流水
            AccountRunningWater accountRunningWater = new AccountRunningWater();
            accountRunningWater.setCompanyId(Long.parseLong(costControl.getCompanyId()));
            accountRunningWater.setAccountManagementId(null);
            accountRunningWater.setStatus("10");// 收入
            accountRunningWater.setBusinessId(costControl.getId());
            accountRunningWater.setAmount(costControl.getAmount());
            accountRunningWater.setCurrency("CNY");
            accountRunningWater.setReceived("20");// 合同到账（10:是，20:否）
            accountRunningWater.setName(costControl.getAccountName());
            accountRunningWater.setOpeningBank(costControl.getAccountBank());
            accountRunningWater.setAccountOpening(costControl.getAccountNumber());
//            accountRunningWater.setType("20"); // 添加流水类型 10 打款  20 到款登记
            accountRunningWater.setTransactionTime(costControl.getApplyTime());
            accountRunningWaterService.save(accountRunningWater);
        }
    }

    /**
     * 重新发起
     */
    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        CostControlDto costControlDto = submitData.toJavaObject(CostControlDto.class);
        costControlService.addOrEdit(costControlDto);
    }

    /**
     * 驳回
     */
    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        costControlService.update(q -> q
                .eq(CostControl::getId, businessId)
                .set(CostControl::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //取消流水结算状态
        costControlService.cancelSettlement(costControlDetailService.list(q -> q.eq(CostControlDetail::getCostControlId, businessId)));
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        costControlService.cancellation(businessId);
    }

    @Override
    public void defaultMethod(Long flowId, Long businessId, NodeTypeEnum handleNodeType, FlowStatusEnum flowStatus, JSONObject submitData) {
        //修改数据
        CostControlDto costControlDto = submitData.toJavaObject(CostControlDto.class);
        if (ObjectUtil.isNotEmpty(costControlDto) && ObjectUtil.isNotEmpty(costControlDto.getId())) {
            costControlService.addOrEdit(costControlDto);
        }

        //执行原本的方法
        super.defaultMethod(flowId, businessId, handleNodeType, flowStatus, submitData);
    }

    @Override
    public List<Long> getHandlingUserList(Long flowId, JSONObject submitData) {
        CostControlDto costControlDto = submitData.toJavaObject(CostControlDto.class);

        List<SysDept> list = sysDeptService.list();
        Map<Long, String> deptMap = list.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        List<Long> uidList = new ArrayList<>();


        for (String companyIdStr : costControlDto.getCompanyId().split(",")) {
            Long companyId = Long.valueOf(companyIdStr);
            String companyName = deptMap.getOrDefault(companyId, companyIdStr);

            List<Long> gmUserList = UserUtil.getUserIdsByRoleKey("GM", companyId);
            if (ObjectUtil.isEmpty(gmUserList)) {
                throw new ServiceException(companyName + ",不存在 总经理角色 用户！");
            }
            uidList.add(gmUserList.get(0));
        }

        return uidList;
    }
}
