package com.fjhx.form.service.cost.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.account.entity.account.po.AccountPayment;
import com.fjhx.account.entity.account.po.AccountSubjects;
import com.fjhx.account.entity.calculate.po.CalculateItem;
import com.fjhx.account.entity.calculate.vo.CalculateItemVo;
import com.fjhx.account.service.account.AccountManagementService;
import com.fjhx.account.service.account.AccountPaymentService;
import com.fjhx.account.service.account.AccountSubjectsService;
import com.fjhx.account.service.calculate.CalculateItemService;
import com.fjhx.account.service.payee.PayeeInfoService;
import com.fjhx.account.service.payment.PaymentTypeService;
import com.fjhx.common.controller.Label;
import com.fjhx.common.entity.art.po.ArtCopyright;
import com.fjhx.common.entity.contractor.po.Contractor;
import com.fjhx.common.entity.logistics.po.LogisticsCompanyInfo;
import com.fjhx.common.entity.patent.po.Patent;
import com.fjhx.common.entity.trademark.po.Trademark;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.art.ArtCopyrightService;
import com.fjhx.common.service.contractor.ContractorService;
import com.fjhx.common.service.logistics.LogisticsCompanyInfoService;
import com.fjhx.common.service.patent.PatentService;
import com.fjhx.common.service.trademark.TrademarkService;
import com.fjhx.common.utils.Assert;
import com.fjhx.customer.entity.customer.po.Customer;
import com.fjhx.customer.service.customer.CustomerService;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.entity.flow.po.FlowExample;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.fjhx.form.entity.cost.dto.CostControlDetailDto;
import com.fjhx.form.entity.cost.dto.CostControlDto;
import com.fjhx.form.entity.cost.dto.CostControlSelectDto;
import com.fjhx.form.entity.cost.po.CostControl;
import com.fjhx.form.entity.cost.po.CostControlDetail;
import com.fjhx.form.entity.cost.vo.CostControlDetailVo;
import com.fjhx.form.entity.cost.vo.CostControlVo;
import com.fjhx.form.mapper.cost.CostControlMapper;
import com.fjhx.form.service.cost.CostControlDetailService;
import com.fjhx.form.service.cost.CostControlService;
import com.fjhx.jushuitan.entity.shop.po.ShopInfo;
import com.fjhx.jushuitan.service.shop.ShopInfoService;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.vo.ContractOutboundInfoVo;
import com.fjhx.sale.service.contract.ContractOutboundInfoService;
import com.fjhx.sale.service.contract.ContractService;
import com.fjhx.supply.entity.supplier.po.SupplierInfo;
import com.fjhx.supply.service.supplier.SupplierInfoService;
import com.fjhx.wms.entity.stock.po.StockJournalDetails;
import com.fjhx.wms.service.stock.StockJournalDetailsService;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 费控 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-18
 */
@Service
public class CostControlServiceImpl extends ServiceImpl<CostControlMapper, CostControl> implements CostControlService {
    @Autowired
    private LogisticsCompanyInfoService logisticsCompanyInfoService;
    @Autowired
    private CostControlDetailService costControlDetailService;
    @Autowired
    private FlowExampleService flowExampleService;
    @Autowired
    private ISysDeptService sysDeptService;
    @Autowired
    private PaymentTypeService paymentTypeService;
    @Autowired
    private StockJournalDetailsService stockJournalDetailsService;
    @Autowired
    private CalculateItemService calculateItemService;
    @Autowired
    private CustomerService customerService;
    @Autowired
    private ContractService contractService;
    @Autowired
    private ContractOutboundInfoService contractOutboundInfoService;
    @Autowired
    private AccountPaymentService accountPaymentService;
    @Autowired
    private SupplierInfoService supplierInfoService;
    @Autowired
    private ShopInfoService shopInfoService;
    @Autowired
    private PayeeInfoService payeeInfoService;
    @Autowired
    private AccountSubjectsService accountSubjectsService;
    @Autowired
    private AccountManagementService accountManagementService;
    @Autowired
    private PatentService patentService;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private ContractorService contractorService;
    @Autowired
    private ArtCopyrightService artCopyrightService;
    @Autowired
    private TrademarkService trademarkService;

    @Override
    public Page<CostControlVo> getPage(CostControlSelectDto dto) {
        IWrapper<CostControl> wrapper = getWrapper();

        wrapper.eq("cc", CostControl::getCostType, dto.getCostType());
        wrapper.eq("cc", CostControl::getStatus, dto.getStatus());
        wrapper.eq("cc", CostControl::getTradeType, dto.getTradeType());

        //到货状态过滤
        wrapper.eq("ap.`status`", dto.getAccountPaymentStatus());

        wrapper.keyword(dto.getKeyword(),
                new SqlField(CostControl::getCode),
                new SqlField(CostControl::getVoucherNo)
        );

        //权限过滤 自己看自己, 财务主管看全部
        List<String> roleKeys = UserUtil.getUserRoleKeys(SecurityUtils.getUserId());
        if (!roleKeys.contains("cfo")) {
            //非财务主管 自己看自己
            wrapper.eq("cc", CostControl::getCreateUser, SecurityUtils.getUserId());
        }

        wrapper.orderByDesc("cc", CostControl::getId);
        Page<CostControlVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<CostControlVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        //赋值主表基础信息
        setInfo(records);
        return page;
    }

    @Override
    public CostControlVo detail(Long id) {
        CostControl costControl = this.getById(id);
        Assert.notEmpty(costControl, "查询不到费控信息");
        CostControlVo result = BeanUtil.toBean(costControl, CostControlVo.class);

        //赋值详情
        setInfo(Arrays.asList(result));

        //赋值明细列表
        setDetailInfo(result);

        //赋值合同信息
        Contract contract = contractService.getById(result.getContractId());
        if (ObjectUtil.isNotEmpty(contract)) {
            result.setContractCode(contract.getCode());
            result.setContractBelongType(contract.getBelongType());
            result.setContractTaxTransportationWay(contract.getTaxTransportationWay());
        }

        //赋值到货信息列表
        List<ContractOutboundInfoVo> contractOutboundInfoList = contractOutboundInfoService.getList(IWrapper.getWrapper().eq(true, "c.id", costControl.getContractId()));
        result.setContractOutboundInfoList(contractOutboundInfoList);

        return result;
    }

    private void setDetailInfo(CostControlVo result) {
        //赋值明细列表
        List<CostControlDetailVo> costControlDetailVos = costControlDetailService.getList(IWrapper.getWrapper().eq(CostControlDetail::getCostControlId, result.getId()));
        if (ObjectUtil.isEmpty(costControlDetailVos)) {
            return;
        }
        //赋值供应商
        supplierInfoService.attributeAssign(costControlDetailVos, CostControlDetailVo::getSupplierId, (item, supplier) -> {
            item.setSupplierName(supplier.getName());
            item.setBusinessName(supplier.getName());
        });
        //赋值快递公司
        logisticsCompanyInfoService.attributeAssign(costControlDetailVos, CostControlDetailVo::getLogisticsCompanyId, (item, logisticsCompany) -> {
            item.setLogisticsCompanyName(logisticsCompany.getName());
            item.setBusinessName(logisticsCompany.getName());
        });
        //赋值店铺
        shopInfoService.attributeAssign(costControlDetailVos, CostControlDetailVo::getShopId, (item, shopInfo) -> {
            item.setShopName(shopInfo.getName());
            item.setBusinessName(shopInfo.getName());
        });
        //赋值收款单位
        payeeInfoService.attributeAssign(costControlDetailVos, CostControlDetailVo::getPayeeAccountId, (item, payeeInfo) -> {
            item.setPayeeAccountName(payeeInfo.getName());
            item.setBusinessName(payeeInfo.getName());
        });
        result.setCostControlDetailList(costControlDetailVos);

        //赋值核算项目
        Map<Long, List<CalculateItemVo>> ciMap = new HashMap<>();
        List<Long> ccdIds = costControlDetailVos.stream().map(CostControlDetail::getId).collect(Collectors.toList());
        List<CalculateItemVo> ciList = calculateItemService.getList(IWrapper.getWrapper().in(CalculateItem::getCostControlDetailId, ccdIds));
        if (ObjectUtil.isNotEmpty(ciList)) {
            //赋值核算项目名称
            setCalculateItemInfo(ciList);

            ciMap = ciList.stream().collect(Collectors.groupingBy(CalculateItem::getCostControlDetailId));
        }
        //赋值记账科目信息
        Map<Long, AccountSubjects> accountSubjectsMap = accountSubjectsService.mapKEntity(AccountSubjects::getId, q -> q.orderByAsc(AccountSubjects::getId));


        for (CostControlDetailVo costControlDetailVo : costControlDetailVos) {
            //赋值核算项目
            costControlDetailVo.setCalculateItemList(ciMap.getOrDefault(costControlDetailVo.getId(), new ArrayList<>()));

            //赋值记账科目树
            int iCount = 0;
            List<String> subjectsNameGroup = new ArrayList<>();
            AccountSubjects accountSubjects = accountSubjectsMap.get(costControlDetailVo.getAccountSubjectsId());
            if (ObjectUtil.isNotEmpty(accountSubjects)) {
                costControlDetailVo.setAccountSubjectsName(accountSubjects.getSubjectsName());
            }
            while (accountSubjects != null && iCount < 50) {
                iCount++;
                subjectsNameGroup.add(0, accountSubjects.getSubjectsName());
                accountSubjects = accountSubjectsMap.get(accountSubjects.getParentId());
            }
            costControlDetailVo.setAccountSubjectsNames(subjectsNameGroup.stream().collect(Collectors.joining(" / ")));
        }
    }

    private void setCalculateItemInfo(List<CalculateItemVo> ciList) {
        for (CalculateItemVo calculateItemVo : ciList) {
            switch (calculateItemVo.getType()) {
                case "部门":
                case "核算中心":
                    SysDept dept = sysDeptService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(dept)) {
                        calculateItemVo.setBusinessName(dept.getDeptName());
                    }
                    break;
                case "专利":
                    Patent patent = patentService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(patent)) {
                        calculateItemVo.setBusinessName(patent.getName());
                    }
                    break;
                case "快递公司":
                    LogisticsCompanyInfo logisticsCompanyInfo = logisticsCompanyInfoService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(logisticsCompanyInfo)) {
                        calculateItemVo.setBusinessName(logisticsCompanyInfo.getName());
                    }
                    break;
                case "股东":
                case "职员":
                    SysUser user = sysUserService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(user)) {
                        calculateItemVo.setBusinessName(user.getUserName());
                    }
                    break;
                case "承包商":
                    Contractor contractor = contractorService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(contractor)) {
                        calculateItemVo.setBusinessName(contractor.getName());
                    }
                    break;
                case "客户":
                    Customer customer = customerService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(customer)) {
                        calculateItemVo.setBusinessName(customer.getName());
                        calculateItemVo.setCustomerName(customer.getName());
                    }
                    break;
                case "店铺":
                    ShopInfo shopInfo = shopInfoService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(shopInfo)) {
                        calculateItemVo.setBusinessName(shopInfo.getName());
                    }
                    break;
                case "供应商":
                    SupplierInfo supplierInfo = supplierInfoService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(supplierInfo)) {
                        calculateItemVo.setBusinessName(supplierInfo.getName());
                    }
                    break;
                case "美术版权":
                    ArtCopyright artCopyright = artCopyrightService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(artCopyright)) {
                        calculateItemVo.setBusinessName(artCopyright.getName());
                    }
                    break;
                case "商标":
                    Trademark trademark = trademarkService.getById(calculateItemVo.getBusinessId());
                    if (ObjectUtil.isNotEmpty(trademark)) {
                        calculateItemVo.setBusinessName(trademark.getName());
                    }
                    break;
                default:
                    calculateItemVo.setBusinessName("未知核算项目！");
            }
        }
    }

    private void setInfo(List<CostControlVo> costControlVos) {
        //赋值收款账户
        accountManagementService.attributeAssign(costControlVos, CostControlVo::getAccountPaymentId, (detail, accountManagement) -> {
            detail.setPaymentAccountManagementName(accountManagement.getName());
        });

        //赋值收付款方式
        paymentTypeService.attributeAssign(costControlVos, CostControlVo::getCostType, (detail, paymentType) -> {
            detail.setCostTypeName(paymentType.getName());
        });
        //赋值物流公司信息
        logisticsCompanyInfoService.attributeAssign(costControlVos, CostControlVo::getLogisticsCompanyId, (detail, logistics) -> {
            detail.setLogisticsName(logistics.getName());
            detail.setLogisticsTaxPoints(logistics.getTaxPoints());
            detail.setLogisticsAccountName(logistics.getAccountName());
            detail.setLogisticsAccountBank(logistics.getAccountBank());
            detail.setLogisticsAccountNumber(logistics.getAccountNumber());
        });
        //赋值申请人信息
        UserUtil.assignmentNickName(costControlVos, CostControlVo::getApplyUserId, CostControlVo::setApplyUserName);
        //归属公司、部门
        List<SysDept> deptList = sysDeptService.list();
        Map<Long, String> deptMap = new HashMap<>();
        if (ObjectUtil.isNotEmpty(deptList)) {
            deptMap = deptList.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));
        }
        for (CostControlVo costControlVo : costControlVos) {
            //归属公司、部门
            List<String> companyNameList = new ArrayList<>();
            for (String companyIdStr : costControlVo.getCompanyId().split(",")) {
                companyNameList.add(deptMap.getOrDefault(Long.valueOf(companyIdStr), companyIdStr));
            }
            costControlVo.setCompanyName(companyNameList.stream().collect(Collectors.joining(",")));
            costControlVo.setDeptName(deptMap.get(costControlVo.getDeptId()));
        }
    }

    @Override
    @DSTransactional
    public void addOrEdit(CostControlDto costControlDto) {

        //保存明细信息
        List<CostControlDetailDto> costControlDetailList = costControlDto.getCostControlDetailList();
        if (ObjectUtil.isEmpty(costControlDetailList)) {
            costControlDetailList = new ArrayList<>();
        }

        //计算总金额
        BigDecimal sumMoney = costControlDetailList.stream()
                .map(CostControlDetail::getMoney)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        costControlDto.setAmount(sumMoney);

        this.saveOrUpdate(costControlDto);


        //删除无效数据
        List<Long> idList = costControlDetailList.stream().map(BaseIdPo::getId).filter(Objects::nonNull).collect(Collectors.toList());
        List<CostControlDetail> delList = costControlDetailService.list(q -> q.eq(CostControlDetail::getCostControlId, costControlDto.getId()).notIn(ObjectUtil.isNotEmpty(idList), BaseIdPo::getId, idList));
        //取消流水结算状态
        cancelSettlement(delList);
        //删除数据
        costControlDetailService.removeBatchByIds(delList);

        for (CostControlDetailDto costControlDetailDto : costControlDetailList) {
            costControlDetailDto.setCostControlId(costControlDto.getId());
            costControlDetailService.saveOrUpdate(costControlDetailDto);
            //保存非发票附件
            ObsFileUtil.editFile(costControlDetailDto.getFileList(), costControlDetailDto.getId(), 0);
            //保存发票附件
            ObsFileUtil.editFile(costControlDetailDto.getInvoiceFileList(), costControlDetailDto.getId(), 10);

            //保存核算项目
            List<CalculateItem> calculateItemList = costControlDetailDto.getCalculateItemList();
            if (calculateItemList == null) {
                calculateItemList = new ArrayList<>();
            }
            calculateItemList.forEach(item -> item.setCostControlDetailId(costControlDetailDto.getId()));
            calculateItemService.editLinked(calculateItemList, CalculateItem::getCostControlDetailId, costControlDetailDto.getId());


            //标记流水为结算中
            String stockJournalDetailsIds = costControlDetailDto.getStockJournalDetailsIds();
            if (ObjectUtil.isNotEmpty(stockJournalDetailsIds)) {
                stockJournalDetailsService.update(q -> q
                        .in(StockJournalDetails::getId, stockJournalDetailsIds.split(","))
                        .set(StockJournalDetails::getSettleStatus, 1));
            }
        }
    }

    @Override
    public void edit(CostControlDto costControlDto) {
        this.updateById(costControlDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public JSONObject getFieldInfo() {
        List<JSONObject> infoField = getField(CostControlVo.class);
        List<JSONObject> detailField = getField(CostControlDetailVo.class);

        JSONObject json = new JSONObject();
        json.put("infoField", infoField);
        json.put("detailField", detailField);


        return json;
    }

    @Override
    public void cancellation(Long id) {
        this.update(q -> q
                .eq(CostControl::getId, id)
                .set(CostControl::getStatus, FlowStatusEnum1.CANCELLATION.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //取消流水结算状态
        cancelSettlement(costControlDetailService.list(q -> q.eq(CostControlDetail::getCostControlId, id)));

        //销毁审批中的流程
        CostControl byId = getById(id);
        flowExampleService.update(q -> q
                .eq(FlowExample::getId, byId.getFlowId())
                .in(FlowExample::getStatus, FlowStatusEnum.READY_START.getKey(), FlowStatusEnum.IN_PROGRESS.getKey())
                .set(FlowExample::getStatus, FlowStatusEnum.CANCELLATION.getKey())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );

        //删除关联打款信息
        accountPaymentService.remove(q -> q.eq(AccountPayment::getCostControlId, id));
    }

    private List<JSONObject> getField(Class cl) {
        //获取所有字段
        List<Field> fieldList = new ArrayList<>();
        Collections.addAll(fieldList, cl.getDeclaredFields());
        Class<? super CostControlVo> superclass = cl.getSuperclass();
        if (ObjectUtil.isNotEmpty(superclass)) {
            Collections.addAll(fieldList, superclass.getDeclaredFields());
        }

        //遍历处理
        List<JSONObject> arr = new ArrayList<>();
        for (Field field : fieldList) {
            Label labelAnnotation = field.getAnnotation(Label.class);
            if (labelAnnotation != null) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("key", field.getName());
                jsonObject.put("val", labelAnnotation.value());
                arr.add(jsonObject);
            }
        }
        return arr;
    }

    /**
     * 取消流水结算状态
     */
    @Override
    public void cancelSettlement(List<CostControlDetail> costControlDetailList) {
        for (CostControlDetail costControlDetail : costControlDetailList) {
            //标记流水为未结算
            String stockJournalDetailsIds = costControlDetail.getStockJournalDetailsIds();
            if (ObjectUtil.isNotEmpty(stockJournalDetailsIds)) {
                stockJournalDetailsService.update(q -> q
                        .in(StockJournalDetails::getId, stockJournalDetailsIds.split(","))
                        .set(StockJournalDetails::getSettleStatus, 0)
                        .set(BasePo::getUpdateTime, new Date())
                        .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                );
            }
        }
    }

}
