package com.fjhx.oa.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.oa.entity.activity.dto.ActivityPriceInventoryDto;
import com.fjhx.oa.entity.activity.po.ActivityPriceInventory;
import com.fjhx.oa.service.activity.ActivityPriceInventoryService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class ActivityPriceInventoryFlow extends FlowDelegate {

    @Autowired
    private ActivityPriceInventoryService activityPriceInventoryService;
    @Autowired
    private CodingRuleService codingRuleService;

    @Override
    public String getFlowKey() {
        return "activity_price_inventory_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        ActivityPriceInventoryDto dto = submitData.toJavaObject(ActivityPriceInventoryDto.class);
        dto.setFlowId(flowId);

        dto.setCode(codingRuleService.createCode(CodingRuleEnum.ACTIVITY_PRICE_INVENTORY.getKey(), null));
        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        activityPriceInventoryService.addOrEdit(dto);

        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        activityPriceInventoryService.update(q -> q
                .eq(ActivityPriceInventory::getId, businessId)
                .set(ActivityPriceInventory::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        start(flowId, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        activityPriceInventoryService.update(q -> q
                .eq(ActivityPriceInventory::getId, businessId)
                .set(ActivityPriceInventory::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        activityPriceInventoryService.cancellation(businessId);
    }
}
