package com.fjhx.oa.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.oa.entity.erp.dto.ErpCodingGroupDto;
import com.fjhx.oa.entity.erp.po.ErpCodingGroup;
import com.fjhx.oa.service.erp.ErpCodingGroupService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class ErpCodingGroupFlow extends FlowDelegate {

    @Autowired
    private ErpCodingGroupService erpCodingGroupService;
    @Autowired
    private CodingRuleService codingRuleService;

    @Override
    public String getFlowKey() {
        return "erp_coding_group_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        ErpCodingGroupDto dto = submitData.toJavaObject(ErpCodingGroupDto.class);
        dto.setFlowId(flowId);

        dto.setCode(codingRuleService.createCode(CodingRuleEnum.ERP_CODING_GROUP.getKey(), null));
        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        erpCodingGroupService.addOrEdit(dto);

        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        erpCodingGroupService.update(q -> q
                .eq(ErpCodingGroup::getId, businessId)
                .set(ErpCodingGroup::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        start(flowId, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        erpCodingGroupService.update(q -> q
                .eq(ErpCodingGroup::getId, businessId)
                .set(ErpCodingGroup::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        erpCodingGroupService.cancellation(businessId);
    }
}
