package com.fjhx.oa.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.oa.entity.medical.dto.MedicalInsuranceDto;
import com.fjhx.oa.entity.medical.po.MedicalInsurance;
import com.fjhx.oa.service.medical.impl.MedicalInsuranceServiceImpl;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class MedicalInsuranceFlow extends FlowDelegate {

    @Autowired
    private MedicalInsuranceServiceImpl medicalInsuranceService;
    @Autowired
    private CodingRuleService codingRuleService;

    @Override
    public String getFlowKey() {
        return "medical_insurance_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        MedicalInsuranceDto dto = submitData.toJavaObject(MedicalInsuranceDto.class);
        dto.setFlowId(flowId);
        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());
        dto.setCode(codingRuleService.createCode(CodingRuleEnum.MEDICAL_INSURANCE.getKey(), null));

        medicalInsuranceService.addOrEdit(dto);

        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        medicalInsuranceService.update(q -> q
                .eq(MedicalInsurance::getId, businessId)
                .set(MedicalInsurance::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        start(flowId, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        medicalInsuranceService.update(q -> q
                .eq(MedicalInsurance::getId, businessId)
                .set(MedicalInsurance::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        medicalInsuranceService.cancellation(businessId);
    }
}
