package com.fjhx.oa.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.oa.entity.seal.dto.SealUseDto;
import com.fjhx.oa.entity.seal.po.SealUse;
import com.fjhx.oa.service.seal.SealUseService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class SealUseFlow extends FlowDelegate {

    @Autowired
    private SealUseService sealUseService;
    @Autowired
    private CodingRuleService codingRuleService;

    @Override
    public String getFlowKey() {
        return "seal_use_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        SealUseDto dto = submitData.toJavaObject(SealUseDto.class);
        dto.setFlowId(flowId);

        dto.setCode(codingRuleService.createCode(CodingRuleEnum.SEAL_USE.getKey(), null));
        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        sealUseService.addOrEdit(dto);

        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        sealUseService.update(q -> q
                .eq(SealUse::getId, businessId)
                .set(SealUse::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        start(flowId, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        sealUseService.update(q -> q
                .eq(SealUse::getId, businessId)
                .set(SealUse::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        sealUseService.cancellation(businessId);
    }
}
