package com.fjhx.oa.service.activity.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.utils.AuthorityUtils;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.entity.flow.po.FlowExample;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.fjhx.jushuitan.service.shop.ShopInfoService;
import com.fjhx.oa.entity.activity.dto.ActivityPriceInventoryDto;
import com.fjhx.oa.entity.activity.dto.ActivityPriceInventorySelectDto;
import com.fjhx.oa.entity.activity.po.ActivityPriceInventory;
import com.fjhx.oa.entity.activity.po.ActivityPriceInventoryDetail;
import com.fjhx.oa.entity.activity.vo.ActivityPriceInventoryDetailVo;
import com.fjhx.oa.entity.activity.vo.ActivityPriceInventoryVo;
import com.fjhx.oa.entity.education.po.EducationSubsidy;
import com.fjhx.oa.mapper.activity.ActivityPriceInventoryMapper;
import com.fjhx.oa.service.activity.ActivityPriceInventoryDetailService;
import com.fjhx.oa.service.activity.ActivityPriceInventoryService;
import com.fjhx.tenant.utils.DeptUstil;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;


/**
 * <p>
 * 活动、价格、库存 服务实现类
 * </p>
 *
 * @author
 * @since 2024-04-10
 */
@Service
public class ActivityPriceInventoryServiceImpl extends ServiceImpl<ActivityPriceInventoryMapper, ActivityPriceInventory> implements ActivityPriceInventoryService {

    @Autowired
    private ShopInfoService shopInfoService;
    @Autowired
    private ActivityPriceInventoryDetailService activityPriceInventoryDetailService;
    @Autowired
    private FlowExampleService flowExampleService;

    @Override
    public Page<ActivityPriceInventoryVo> getPage(ActivityPriceInventorySelectDto dto) {
        IWrapper<ActivityPriceInventory> wrapper = getWrapper();

        //权限过滤
        AuthorityUtils.cofAuthority("api", ActivityPriceInventory::getCompanyId, wrapper);

        wrapper.keyword(dto.getKeyword(),
                new SqlField("api", ActivityPriceInventory::getCode),
                new SqlField("api", ActivityPriceInventory::getActivityName)
        );
        wrapper.eq("api", EducationSubsidy::getStatus, dto.getStatus());

        wrapper.orderByDesc("api", ActivityPriceInventory::getId);
        Page<ActivityPriceInventoryVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ActivityPriceInventoryVo> records = page.getRecords();
        setInfo(records);
        return page;
    }

    @Override
    public ActivityPriceInventoryVo detail(Long id) {
        ActivityPriceInventory ActivityPriceInventory = this.getById(id);
        ActivityPriceInventoryVo result = BeanUtil.toBean(ActivityPriceInventory, ActivityPriceInventoryVo.class);
        List<ActivityPriceInventoryDetailVo> list = activityPriceInventoryDetailService.getList(IWrapper.getWrapper().eq("apid.id", result.getId()));
        result.setActivityPriceInventoryDetailList(list);
        return result;
    }

    private void setInfo(List<ActivityPriceInventoryVo> records) {
        if (ObjectUtil.isEmpty(records)) {
            return;
        }
        //中心、部门、申请人
        UserUtil.assignmentNickName(records, ActivityPriceInventoryVo::getCreateUser, ActivityPriceInventoryVo::setCreateUserName);
        DeptUstil.assignmentNickName(records, ActivityPriceInventoryVo::getDeptId, ActivityPriceInventoryVo::setDeptName);
        DeptUstil.assignmentNickName(records, ActivityPriceInventoryVo::getCompanyId, ActivityPriceInventoryVo::setCompanyName);
        //店铺名称
        shopInfoService.attributeAssign(records, ActivityPriceInventory::getShopId, (item, shop) -> {
            item.setShopName(shop.getName());
        });
        //支持方
        DeptUstil.assignmentNickName(records, ActivityPriceInventoryVo::getSupporterId, ActivityPriceInventoryVo::setSupporterName);
    }

    @DSTransactional
    @Override
    public void addOrEdit(ActivityPriceInventoryDto dto) {
        this.save(dto);
        List<ActivityPriceInventoryDetail> activityPriceInventoryDetailList = dto.getActivityPriceInventoryDetailList();
        activityPriceInventoryDetailList.stream().forEach(item -> item.setActivityPriceInventoryId(dto.getId()));
        activityPriceInventoryDetailService.editLinked(activityPriceInventoryDetailList, ActivityPriceInventoryDetail::getActivityPriceInventoryId, dto.getId());
        ObsFileUtil.editFile(dto.getSupporterFileList(), dto.getId(), 10);
        ObsFileUtil.editFile(dto.getStockFileList(), dto.getId(), 20);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @DSTransactional
    @Override
    public void cancellation(Long businessId) {
        ActivityPriceInventory byId = getById(businessId);
        this.update(q -> q
                .eq(ActivityPriceInventory::getId, businessId)
                .set(ActivityPriceInventory::getStatus, FlowStatusEnum1.CANCELLATION.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //销毁审批中的流程
        flowExampleService.update(q -> q
                .eq(FlowExample::getId, byId.getFlowId())
                .in(FlowExample::getStatus, FlowStatusEnum.READY_START.getKey(), FlowStatusEnum.IN_PROGRESS.getKey())
                .set(FlowExample::getStatus, FlowStatusEnum.CANCELLATION.getKey())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );
    }

}
