package com.fjhx.oa.service.education.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.utils.AuthorityUtils;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.entity.flow.po.FlowExample;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.fjhx.oa.entity.education.dto.EducationSubsidyDto;
import com.fjhx.oa.entity.education.dto.EducationSubsidySelectDto;
import com.fjhx.oa.entity.education.po.EducationSubsidy;
import com.fjhx.oa.entity.education.vo.EducationSubsidyVo;
import com.fjhx.oa.mapper.education.EducationSubsidyMapper;
import com.fjhx.oa.service.education.EducationConfigService;
import com.fjhx.oa.service.education.EducationSubsidyService;
import com.fjhx.tenant.utils.DeptUstil;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;


/**
 * <p>
 * 学历补贴申请 服务实现类
 * </p>
 *
 * @author
 * @since 2024-04-08
 */
@Service
public class EducationSubsidyServiceImpl extends ServiceImpl<EducationSubsidyMapper, EducationSubsidy> implements EducationSubsidyService {

    @Autowired
    private FlowExampleService flowExampleService;
    @Autowired
    private EducationConfigService educationConfigService;

    @Override
    public Page<EducationSubsidyVo> getPage(EducationSubsidySelectDto dto) {
        IWrapper<EducationSubsidy> wrapper = getWrapper();

        //权限过滤
        AuthorityUtils.cofAuthority("es", EducationSubsidy::getCompanyId, wrapper);

        wrapper.keyword(dto.getKeyword(), new SqlField("es", EducationSubsidy::getCode));
        wrapper.eq("es", EducationSubsidy::getStatus, dto.getStatus());

        wrapper.orderByDesc("es", EducationSubsidy::getId);
        Page<EducationSubsidyVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<EducationSubsidyVo> records = page.getRecords();
        setInfo(records);
        return page;
    }

    @Override
    public EducationSubsidyVo detail(Long id) {
        EducationSubsidy EducationSubsidy = this.getById(id);
        EducationSubsidyVo result = BeanUtil.toBean(EducationSubsidy, EducationSubsidyVo.class);
        setInfo(Arrays.asList(result));

        return result;
    }

    void setInfo(List<EducationSubsidyVo> records) {
        if (ObjectUtil.isEmpty(records)) {
            return;
        }
        educationConfigService.attributeAssign(records, EducationSubsidyVo::getEducationId, (item, education) -> {
            item.setEducationName(education.getName());
        });

        UserUtil.assignmentNickName(records, EducationSubsidyVo::getCreateUser, EducationSubsidyVo::setCreateUserName);
        DeptUstil.assignmentNickName(records, EducationSubsidyVo::getDeptId, EducationSubsidyVo::setDeptName);
        DeptUstil.assignmentNickName(records, EducationSubsidyVo::getCompanyId, EducationSubsidyVo::setCompanyName);
    }

    @DSTransactional
    @Override
    public void addOrEdit(EducationSubsidyDto educationSubsidyDto) {
        this.saveOrUpdate(educationSubsidyDto);
        ObsFileUtil.editFile(educationSubsidyDto.getFileList(), educationSubsidyDto.getId());
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @DSTransactional
    @Override
    public void cancellation(Long businessId) {
        EducationSubsidy byId = getById(businessId);
        this.update(q -> q
                .eq(EducationSubsidy::getId, businessId)
                .set(EducationSubsidy::getStatus, FlowStatusEnum1.CANCELLATION.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //销毁审批中的流程
        flowExampleService.update(q -> q
                .eq(FlowExample::getId, byId.getFlowId())
                .in(FlowExample::getStatus, FlowStatusEnum.READY_START.getKey(), FlowStatusEnum.IN_PROGRESS.getKey())
                .set(FlowExample::getStatus, FlowStatusEnum.CANCELLATION.getKey())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );
    }

}
